// midi.h library info http://playground.arduino.cc/Main/MIDILibrary
// MIDI_Class Class Reference http://arduinomidilib.sourceforge.net/a00001.html

boolean windMode = false;
byte lastNote = 0;

void HandleNoteOn(byte channel, byte pitch, byte velocity) 
{ 
  // Do whatever you want when you receive a Note On.
  // Try to keep your callbacks short (no delays ect) as the contrary would slow down the loop()
  // and have a bad impact on real-time performance.
  if (channel == triggerChannel && pitch < 112) // receive patch changes on triggerChannel (default channel 10); there are only 64 memory locations
  {      
    if (checkToc(pitch))
    {
      recall = true; // we're ready to recall a preset
      pageNum = pitch * 4; // one memory location is 4 pages long
      cued = pitch; // the pattern that will play next is the value of "pitch" - need this to blink lights
      head = pitch; // need this for pattern chaining to work properly
      buttonMode = 4; // make sure we're in trigger mode, if we weren't already
      trigPage = pitch/32; // switch to the correct trigger page
      controlLEDrow = B00000001; // light the LED for trigger mode
      controlLEDrow = bitSet(controlLEDrow, trigPage + 1); // light the LED for the page we're on     
    }
  }
  else if (buttonMode != 7  && pitch > 0)
  {
    if (!seqRunning && midiNoteEntry)
    { 
      if (velocity != 0) // some instruments send note off as note on with 0 velocity
      {
        seqNotes[seqCurrentStep] = pitch;
        seqCurrentStep = (seqCurrentStep + 1) % 32;
        noteOn = true;
        syncPhaseInc = pgm_read_word_near(midiTable + pitch);
        lastNote = pitch;
        if (!windMode)
        {
          grainPhaseInc = mapPhaseInc(512 - (pot1 >> 1))/ 2;  
          int newVelocity = (pot3 + velocity < 1023) ? pot3 + velocity : 1023; 
          grain2PhaseInc = mapPhaseInc(1023 - newVelocity) / 2;
        }
        automate = false;
        //delay(2);
      }
      else // velocity = 0;
      {
        if (!windMode && pitch == lastNote) 
          noteOn = false;
      }
    }
    else if (!midiNoteEntry)// the sequencer is running
    {     
      if (pitch != 60) //middle C / C4 = note number 60
          transpose(33, pitch - 60);
    }
  }
}

void HandleNoteOff(byte channel, byte pitch, byte velocity) 
{
  if (pitch == lastNote)
    noteOn = false;
} 

void HandleCC(byte channel, byte number, byte value)
{
  if(!seqRunning && number == 2)
  {
    windMode = true;
    if (value == 0)
      noteOn = false;
    else
      grainPhaseInc = mapPhaseInc(512 - (value << 2))/ 2;
    grain2PhaseInc = mapPhaseInc(512 - (value << 2)) / 2;
    //delay(2);  
  } 
}

void HandleClock(void) // what to do for 24ppq clock pulses
{
  midiClock = true; // to fix a situation where we don't hear notes because seqRunning is not true
  seqRunning = false;
  pulse++; // 6 for a 16th
  autoCounter = (autoCounter < 191) ? autoCounter + 1 : 0; 
  static boolean long16th = true; // is this the first/longer 16th of the swing pair?
  char swingPulse = swing / 86; // to give us some swing when synced to midi clock (sadly only 2 levels: full swing and half swing)
  // takes a value between 0 and 255 and returns 0, 1 or 2
  swingPulse = (!long16th) ? (0 - swingPulse) : swingPulse;                              
  if (pulse >= (6 + swingPulse))
  {
    currentTime = millis();
    pulse = 0;
    seqMidiStep = true;
    seqTrueStep = (seqTrueStep+1)%32; 
    if (seqLength == 32)
      seqCurrentStep = seqTrueStep;
    else
    {
      seqCurrentStep = (seqCurrentStep + 1)%seqLength;
    }
    long16th = !long16th;

    sixteenthStartTime = millis();
  }
}

void HandleStart (void)
{
  seqTrueStep = 0;
  seqCurrentStep = 0;
  autoCounter = 0;
  sixteenthStartTime = 0;
}

void HandleStop (void)
{
  midiClock = false;
  seqTrueStep = 0;
  seqCurrentStep = 0;
  autoCounter = 0;
  sixteenthStartTime = 0;
  if (midiNoteOut)
  {
    for (byte i = 0; i < 128; i++) // turn off all midi notes
      MIDI.sendNoteOff(i, 0, noteChannel);
  }
  seqRunning = false;
  noteOn = false; 
}













